function [xBest, fBest,FEs, intermed] = ...
  APS11 (funSet, fun, FEmax, opt_f,FuncAddr,LB,UB, expAlpha,contractGamma, ...
  restartRate,strategArray,N,verbose,FEintermed, keepRate, triosAll,quantis)
%{
 Adaptive Population-based Simplex. Portfolio version
 Proposed by: Maurice Clerc and Mahamed G.H. Omran
 Version: 2016-04-12

 Inputs
		See APS11_Monte

 Outputs
		xBest = position of the best agent
   fBest = best solution found (minimum value)
   FEs = number of FEs usable to solve the function "fun" (if possible)
		intermed= intermediate best values, for information and evolution curves

 Main differences from APS9
 - stops to search exactly when the maximum number of FEs is reached
  (it may be important when FEmax is small)
	- the local search has been modified in order to generate less duplicates
 - instead of handling duplicates after they have been all generated,
		removes each one immediately. The process is called "increasing diversity".
 - the search space is systematically normalised (as [0,1]^D), so that
 	the diameter of the population is meaningful (see probaV())
	- can cope with quantified variables (typically integer ones).
%}

% Maximum number of strategies to use	during the search
strategNb=length(strategArray);
searchStat=zeros(2,10); % To save statistics about the search

% If restart, number of agents to reinitialise
Ninit=max(1,round(restartRate*N)); % At least 1

%------------------------------------------------------------------ Run  --    1
x=[];
f=[];

%		Initialises the positions
%	Note they are sorted by increasing f
FEs=0;

[x,f,FEs]=initRand(N,LB,UB,x,f,fun,opt_f,FuncAddr,FEs,quantis);

% 	Saves the current best result (value and position)
fBest=f(1);
xBest=LB+(UB-LB).*x(1,:);
fprintf('\n Min after init: %1.4e',fBest);

%		In case of automatic restarts, counts them for information
restartNb=0;


%------------------------------------------------------------ Main loop -- 1.1
intermed=Inf*ones(1,length(FEintermed));
s=1; % Starts with the first strategy
iter=0;

while FEs<=FEmax
  if verbose>0 fprintf('\n FE %i,  min %1.4e',FEs, f(1)); end
iter=iter+1;   
  % ------------------------------- Successively tries the strategies -- 1.1.1
  fMinPrev=f(1); % Best value before moves (remember f is sorted)
  FEprev=FEs; % Number of FEs already used
 %fprintf('\n FEs before strategy %i: %i',s, FEs) 
  [improv,FEs,searchStat,x,f]= ...
    strategApply(x,f, s,strategArray, expAlpha,contractGamma, ...
    fun, opt_f, LB, UB,FuncAddr, FEs, verbose, searchStat,...
    keepRate, triosAll,FEmax,quantis);
  %		fprintf(' and after: %i', FEs) 
    if ~improv
      if s==strategNb s=1; else s=s+1; end
    end
    
    % 	If partial restart method is used, tries to detect stagnation
    Nnew=min(Ninit,FEmax-FEs);
    if Nnew>0 && restartRate>0
      stagn=stagnation(fMinPrev,f(1),FEprev,FEs);
      
      if stagn==1 % In case of probable stagnation, ...
        if FEs<=FEmax-Nnew % To be sure to no exceed the budget, even a bit
        	restartNb=restartNb+1;
        	% Random restart
        	[x,f,FEs]= initRand(Nnew,LB,UB,x,f,fun,opt_f,FuncAddr, FEs,quantis);      
      end
      end
    end
    
    % Saves intermediate results
    Ind=find(FEintermed<=FEs);
    indL=length(Ind);
    interm=max(1,indL);
    intermed(interm)=min(f);
    
   
end %  while FEs<=FEmax ------------------------------ End of main loop (1.1)



% Saves the best result (value and position)
fBest=f(1);
xBest=LB+(UB-LB).*x(1,:); 

% Prints the result
fprintf('\n  %i iterations',iter); 
fprintf('\n 	  %i	FEs =>	 %1.4e', FEs,fBest);

if verbose>0
  fprintf('\nxBest');disp(xBest)
end

%		Prints final information
fprintf('\nPopulation size %i',N);

% 	Some stats
fprintf('\nStrategy Nb Success Rate');
for s=1:length(strategArray)
  str=strategArray(s);
  fprintf('\n%i       %i   %i   %f',str, searchStat(1,s),...
    searchStat(2,s),searchStat(2,s)/searchStat(1,s));
end

if restartRate>0
  fprintf('\n  Stagnations  %i',restartNb);
end

end % End of APS
%====================================================================== initRand
function 	[x,f, FEs]=initRand(Nnew,LB,UB,x0,f0,fun,opt_f,FuncAddr,FEs,quantis)
%	Nnew= number of positions to (re)initialise
D=length(LB);
N=length(f0); % Is null at the very beginning

%		Initialisation and evaluation
%{
 At the very beginning, we know nothing except the search space.
In such a case, it is easy to prove that the best very first point must be
the centre of the search space
%}

if N==0; xNew(1,1:D)=0.5*ones(1,D); 
		begin=2;
	else
		begin=1;	
end

	for n =begin:Nnew % All variables are first supposed continuous
		xNew(n,1:D)=rand(1,D);
	end

if  ~isempty(quantis) % If some are discrete, it may be necessary to modify them
	for n=1:Nnew
		xNew(n,:)=quantify(xNew(n,:),quantis);
	end
end
		
for n = 1 : Nnew
  [fNew(n), FEs]=fEval(	xNew(n,:), fun, opt_f,FuncAddr,FEs, LB, UB,[]);
end

% Completes, if just partial reinitialisation
if N>Nnew
  nOld=N-Nnew;
  x=[x0(1:nOld,1:D); xNew(:,1:D)];
  f=[f0(1:nOld) fNew];
else
  x=xNew;
  f=fNew;
end

% Sorts (increasing order)
[f,Ind]=sort(f);
x=x(Ind,:);

end

%========================================================================= fEval
function [fx,FEs]=fEval(xNorm, fun, opt_f,FuncAddr,FEs, LB, UB,quantis)
global keepAll
global xTot % TEST, signature
global fTot  % TEST, entropy
global savePos
global signature

if ~isempty(quantis)
	xNorm=quantify(xNorm,quantis);
end

x=LB+(UB-LB).*xNorm;
fx = FuncAddr(x, fun, opt_f);

% Unfortunately, there are  sometimes bugs in function evaluations
% and the position can not be evaluated
%  (see for example CEC 2011 competition, functions 5 and 6)
if isnan(fx)
  fx=10^10; % Inf can not be used, for fmincon does not accept it
else
  FEs=FEs+1;
end

 if savePos || signature
   	xTot=[xTot;x];
  end % Just for tests
   
  if savePos
    fTot=[fTot fx]; % TEST, entropy
  end

end
%================================================================== strategApply
function [improv, FEs,searchStat,x,f]=...
  strategApply(x,f,s,strategArray,expAlpha,contractGamma, ...
  fun, opt_f, LB,UB, FuncAddr,FEs,verbose, searchStat,...
  keepRate, triosAll,FEmax,quantis)
 global zerosD onesD
 
 contract=contractGamma;

[N,D]=size(x);
Ntemp=min(N,FEmax-FEs);

% We will keep the eliteSize best positions for the next iteration
keepSize=max(1,round(N*keepRate));
if keepSize+Ntemp<N % Not enough budget for more search
  %											we stop here
  improv=0;
  FEs=FEmax+1;
  return;
end

strategNb=length(strategArray);
strateg=strategArray(s);

searchStat(1,s)=searchStat(1,s)+1; % Counts, for information

% Modifies the probability threshold
pv=probaV(x);
toEval=ones(1,N);

switch strateg
  case 1  % Probabilistic expansion, dimension-wise
    trios=triplet(N,N,triosAll);
    
    for n=1:Ntemp
      r1=trios(n,1); r2=trios(n,2); r3=trios(n,3);
      
      for d=1:D
        if rand < pv % Semi-reflection of the worst
          xTemp(n,d)=x(r3,d)+(1+expAlpha)*((x(r1,d)+x(r2,d))/2-x(r3,d));
        else
          xTemp(n,d)=x(n,d);
        end	%	if rand < pv
      end	%	 for d=1:D
      
      % It might happen that the position is not modified
      % In that case, we just "look around"
      if xTemp(n,:)==x(n,:)
        xTemp(n,:)=around(x(n,:),N);
        
      else % Checks confinement, and possibly adjusts the position;
        xTemp(n,:) = handBound(xTemp(n,:));
      end
   
    end	%	for n=1:N
    
  case 2 % Probabilistic contraction, dimension-wise
    trios=triplet(N,N,triosAll);
    %nPerm=randperm(Ntemp);
    
    %for n0=1:Ntemp
    for n=1:Ntemp
    	%n=nPerm(n0);
      r1=trios(n,1); r2=trios(n,2); r3=trios(n,3);
      
      for d=1:D
        if rand < pv
        if contractGamma<0 % To reduce bias. Note this is not always
      											% really "contraction"
     				contract=2*rand;
     		end
     		  
          c=bellLike(0.5, 3,3); %Three random numbers from a bell-like distribution
          c=c/sum(c); % => sum(c)=1
          xTemp(n,d)=(c(1)*x(r1,d)+c(2)*x(r2,d)+c(3)*x(r3,d))*contract; %1/3;
         %{
          Might be outside the search space, because "contract"
          On the other hand, with the true weighted gravity center,
         	results are not as good.
       %}
        else
          xTemp(n,d)=x(n,d);
        end % if rand < pv
      end % for d=1:D
      
      % It might happen that the position is not modified
      if xTemp(n,:)==x(n,:)
        xTemp(n,:)=around(x(n,:),N);
        
      else % Checks confinement, and possibly adjusts the position;
        xTemp(n,:) = handBound(xTemp(n,:));
      end
   
    end % for n=1:N
    
  case {3 4}  % Local search. Tries to improve the best
    
    MaxFunEvals=min(FEmax-FEs,3*N*D); % Empirical
   ff = @(u)fEval(u,fun,opt_f,FuncAddr,FEs, LB, UB,quantis);
      TolCon=0;  TolFun=0; TolX=0;
 %fprintf('\n  MaxFunEvals %i', MaxFunEvals);  
    switch strateg
    case 3 % fmincon.  Interior point method (Matlab Optimization toolbox)
    %  http://fr.mathworks.com/help/optim/ug/constrained-nonlinear-optimization-algorithms.html
    
    options = optimoptions(@fmincon,'MaxFunEvals',MaxFunEvals,...
    'Display','off','Algorithm', 'interior-point', ...
    'TolCon',TolCon, 'TolFun',TolFun);

    [xTemp(1,:),fTemp(1),~,output] = fmincon(ff,x(1,:),[],[],[],[],...
   																					zerosD,onesD,[],options);
   	case 4 % TEST.  minimize
   	% Minimizes constrained functions with FMINSEARCH or FMINLBFGS
   	% http://www.mathworks.com/matlabcentral/fileexchange/24298-minimize
   	
   	 options = setoptimoptions('MaxFunEvals',MaxFunEvals,...
    'display','off','Algorithm','fminlbfgs', ...
    'TolCon',TolCon, 'TolFun',TolFun, 'TolX', TolX);
    % Note: Instead of fminlbfgs, one can use fminsearch
    
   [xTemp(1,:),fTemp(1),~,output] =minimize(ff,x(1,:),[],[],[],[],...,
   																	zerosD,onesD) %,options);
    end
    
% fprintf('\n  fmincon, %i FEs',output.funcCount )   
    FEs=FEs+ output.funcCount;
    
    if FEs>=FEmax % Not enough budget for more search, we stop here
      improv=0; FEs=FEmax+1;
      return;
    end
    
    toEval(1)=0; % xTemp(1) has been evaluated
    
    % Increases diversity
    % For other positions (and even 1 if the above local search did not
    % improve it, looks "around"
    
    if fTemp(1)>=f(1)
      xTemp(1,:)=around(x(1,:),N);
      toEval(1)=1;
    end
    
    k=1;
    for n=keepSize+1:N % The worst ones
      k=k+1;
      xTemp(k,:)=around(x(n,:),N);
    end
    Ntemp=k;
    
end % switch strateg

% Quantifies xTemp (if needed)
if ~isempty(quantis)
	for n=1:Ntemp
		xTemp(n,:)=quantify(xTemp(n,:),quantis);
	end
end

%---------------Evaluates xTemp  (the new positions)
for n=1:Ntemp
  if toEval(n)==1
    [fTemp(n), FEs]=fEval(xTemp(n,:), fun, opt_f,FuncAddr,FEs, LB, UB,[]);
  end
end

%	Adds the keepSize best ones
xAll=[x(1:keepSize,:);xTemp];
fAll=[f(1:keepSize) fTemp];

% Sorts according to the fitness
[fAll, Ind]=sort(fAll);
xAll=xAll(Ind,:);

% Keeps the N best ones for the next iteration
fPrev=f;
f=fAll(1:N);
x(1:N,:)=xAll(1:N,:);

%	Estimates the number of improvements
% Note that it is usually not the exact count, for we did not memorise the ranks of
% the previous positions and f has been re-sorted
improvNb=0;
for n=1:N
  if f(n)<fPrev(n) improvNb=improvNb+1; end 
end
%fprintf('\n improvNb %i',improvNb)
% Checks if "enough" improvement
if improvNb>keepSize
  improv=true;
  searchStat(2,s)=searchStat(2,s)+1;
else
  improv=false;
end

end % End of strategApply
%========================================================== Stagnation detection
function stagn=stagnation(fMinPrev,fMin, FEprev, FEs)
% 	Normalised pseudo-gradient of the fitness evolution
delta=2*(fMinPrev-fMin)/abs(fMinPrev+fMin);
delta=delta/(FEs-FEprev);
expBeta=1;
stagProba=expBeta*exp(-delta);
stagn= rand<stagProba;
end
%======================================================= Probabilistic threshold
function pv=probaV(x)
global diagonal

% Compares to the search space
    diameter=max(pdist(x));
    pv=diameter/diagonal;
end
%=================================================================== handBound
function x=handBound(x)
%	Keeps the position inside the normalised search space
D=length(x); % Dimension

for d=1:D
  if x(d)<0 || x(d)>1
    x(d)=rand;
  end
end
end
%====================================================================  triplet
function trios=triplet(nb,N, triosAll)
% Generates nb different sorted trios  from {1, 2, ..., N}.    .

%	 triosAll contains all possible trios, already sorted.
%		We select at random nb trios amongst the K first ones.
%		As triosAll is sorted in lexicographic order, the first ones are also the
% 	best ones

K=N+nb;
triosTemp=triosAll(1:K,:);
nbTrios=K;

for n=1:nb
  %	Selects at random a trio
  ik=randi(nbTrios);
  trios(n,:)=triosTemp(ik,:);
  
  %	Removes the selected trio, to be sure to not resample it
  triosTemp(ik,:)=[]; nbTrios=nbTrios-1;
end
end % End of triplet()
%======================================================================== around
function xAround=around(x,N)
% Samples a position "around" a given one
	D=length(x);
  sigma=0.5/N; % Empirical
  nSum=5; % 5
  
	for d=1:D 
 		c=bellLike(-0.5,1,nSum); g=sigma*c; % J.  Bell-like perturbation        
  	xAround (d) = min(1,max(0,x(d)+g));
	end
end
%====================================================================== bellLike
function rnd=bellLike(shift, nRnd, nSum)
%	Sample nRnd random numbers from a bell-like distribution
% The higher nSum, the more similar to the Normal distribution
% The support is [shift, 1+shift]
% The mean is shift+0.5

	 rnd=zeros(1,nRnd);  
    for k=1:nSum 
    	rnd=rnd+rand(1,nRnd);
    end
    rnd=shift+rnd/nSum;    
end
%====================================================================== quantify
function x=quantify(x,quantis)
	D=length(x);
	for d=1:D
		if quantis(d)>0
			x(d)=quantis(d)*round(x(d)/quantis(d));
		end
	end
end
